## fd_aws_deployer_sandbox

### General notes

* To see more detailed logs modify log levels in the relevant `log4j2.xml` file.


### Load the project into IntelliJ

1. Make sure you have [Gradle](https://gradle.org/) installed.

2. Clone the project.

3. If applicable: Checkout the branch you intend to work on.

4. Pull the source code of [Git submodules](https://git-scm.com/book/en/v2/Git-Tools-Submodules):

   `git submodule update --init --recursive`

   The Git submodules are now in a detached head state.  In other words: the Git submodules are pointing to a specific commit in the encapsulated projects.  If you want them to point to the `master` branch you can execute the following:

   `git submodule foreach --recursive git checkout master`

5. Import the project into your IDE:
    * IntelliJ:
    
        * Execute in the project root:
        
          `gradlew openIdea`

6. Within IntelliJ, confirm that everything is okay:
    * Confirm that the project contains modules, not folders.  If IntelliJ
      doesn't correctly reflect the module structure the project might not have
      compiled.
    
    * If you are having problems tunneling into Java source code
      (eg `java.lang.String`) it might be indicative of an IntelliJ configuration
      error.
      
      Some IntelliJ configuration you can look at:
      
      * Configured Java SDKs:
      
        `[File] > [Project Structure]`, `[Platform Settings] > [SDKs]`
      
      * Java SDK configured for use by Gradle:
        
        `[File] > [Settings]`, `[Build, Execution, Deployment] > [Build Tools] > [Gradle]`.


### Using the AWS API externally

* Build a fat jar of the AWS API module as follows:
  
  `gradlew :shared:aws-api:shadowJar`
  
  Navigate to the `:shared:aws-api` module's `build\libs` folder.
  
  The fat jar is the jar with the `-all.jar` suffix, for example: `aws-api-1.0.0-all.jar`.


### Testing

#### Integration tests

##### Integration tests - General mechanism

* Integration tests are managed in the `src/integration-test` folder of relevant modules.

* The Gradle build has been configured to not execute integration tests by default.

* The build server will execute integration tests.

* If you really need to execute integration tests locally, execute:
  `gradlew integrationTest`. 

##### Integration tests - Configuring access credentials

* Configure your AWS credentials in `AwsTestProfile`.


### Deploying examples within this project

#### Deploying from Java

* Look at some of the integration tests.

* Simplest example: `BasicExampleCloudformationStack_IntegrationTest`

#### Deploying from the command line

* Deploy using Gradle.

* Simplest example: See module `:cloudformation-deployments:basic-example`

* Note the Powershell wrapper for the Gradle build file.

* Configure your AWS credentials in `deploy.properties`.


### Errata

* To check dependencies for security issues execute:
  `gradlew dependencyCheckAnalyze`

